""" Importing tests here means they are found for running test oraclepool
    Need to also add each as an app for it to have the test models needed
    eg. oraclepool.tests.performance
"""
import monkeypatches
from oraclepool.tests.aggregates import models
from oraclepool.tests.apitest.tests import test_dbapi
from oraclepool.tests.regress.tests import *
from oraclepool.tests.nulls.tests import NullTests
from oraclepool.tests.slicing.tests import PagingTestCase, DistinctTestCase
from oraclepool.tests.performance.tests import PerformanceTestCase





from decimal import Decimal
# Internal dbapi module 
import  cx_Oracle
# Base unit test - Python database API 2.0 standard
import dbapi20
from oraclepool.tests.settings import get_settings_dict
settings_dict = get_settings_dict()

if settings_dict['ENGINE'] == 'oraclepool':
    from oraclepool.base import DatabaseWrapper
else:
    from django.db.backends.oracle.base import DatabaseWrapper

print 'Database engine %s' % settings_dict['ENGINE']

def connect_string():
    """ Lifted from oracle base """
    if len(settings_dict['HOST'].strip()) == 0:
        settings_dict['HOST'] = 'localhost'
    if len(settings_dict['PORT'].strip()) != 0:
        dsn = cx_Oracle.makedsn(settings_dict['HOST'],
                               int(settings_dict['PORT']),
                               settings_dict['NAME'])
    else:
        dsn = settings_dict['NAME']
    return "%s/%s@%s" % (settings_dict['USER'],
                         settings_dict['PASSWORD'], dsn)


class test_dbapi(dbapi20.DatabaseAPI20Test):
    """ The default test case will just run against the driver
        TODO: Populate the subclassed methods below to duplicate 
        driver tests with wrapper based equivalent tests
    """
    driver = cx_Oracle
    wrapper = DatabaseWrapper(settings_dict)
    connect_args = [ connect_string() ]
    
    def _try_run(self, *args):
        cur = self.wrapper._cursor
        try:
            cur = con.cursor()
            for arg in args:
                cur.execute(arg)
        finally:
            try:
                if cur is not None:
                    cur.close()
            except: pass
            con.close()

    def _try_run2(self, cur, *args):
        for arg in args:
            cur.execute(arg)
        
    def test_select_decimal_zero(self):
        """ Test decimal zero
            TODO: Sort out for decimals and get parsing to work
            as expected, currently does same as oracle engine
        """
        expected = (
            float('0.01'),
            float('0.0'),
            float('-0.0'))

        cur = self.wrapper._cursor()
        cur.execute("SELECT %s as A, %s as B, %s as C from dual", expected)

        result = cur.fetchall()
        self.assertEqual(result[0], tuple([unicode(f) for f in expected]))
        self.wrapper.connection.close()

    # not implemented; use a string instead
    def test_Binary(self):
        pass

    # not implemented; see cx_Oracle specific test suite instead
    def test_callproc(self):
        pass

    # not implemented; Oracle does not support the concept
    def test_nextset(self):
        pass

    # not implemented; see cx_Oracle specific test suite instead
    def test_setinputsizes(self):
        pass

    # not implemented; see cx_Oracle specific test suite instead
    def test_setoutputsize(self):
        pass

    # Other cx_Oracle failing tests skipped
    def test_retval(self):
        pass

    def test_fetchmany(self):
        pass

    def test_rowcount(self):
        pass
    
    def test_Time(self):
        pass

    def test_ExceptionsAsConnectionAttributes(self):
        pass



import datetime
import decimal
from django.db import models, IntegrityError
from django.core.paginator import Paginator

class Bug19Table(models.Model):
    """ A simple model for testing string comparisons.
    
    >>> choices = (
    ...     "no_slashes", "no_slashes_2",
    ...     r"C:\some_folder\somefile.ext", r"\some_folder",
    ...     "some_folder"+'\',
    ...     "[brackets]",
    ...     )
    >>> for c in choices:
    ...     Bug19Table.objects.create(choice=c).save()
    >>> len(Bug19Table.objects.all())
    6    
    >>> len(Bug19Table.objects.filter(choice__contains="shes"))
    2
    >>> len(Bug19Table.objects.filter(choice__endswith="shes"))
    1
    >>> len(Bug19Table.objects.filter(choice__contains=r"der\som"))
    1
    >>> len(Bug19Table.objects.filter(choice__endswith=r"der\somefile.ext"))
    1
    >>> len(Bug19Table.objects.filter(choice__contains="["))
    1
    """
    choice = models.TextField(max_length=200)
    
    def __unicode__(self):
        return self.choice


class Bug21Table(models.Model):
    """
    Test adding decimals as actual types or as strings.
    
    >>> Bug21Table(a='decimal as decimal', d=decimal.Decimal('12.34')).save()
    >>> Bug21Table(a='decimal as string', d=u'56.78').save()
    >>> len(list(Bug21Table.objects.all()))
    2
    """
    a = models.CharField(max_length=50)
    d = models.DecimalField(max_digits=5, decimal_places=2)

class Bug27Table(models.Model):
    """
    Test that extra/select works, and doesn't interfere with the 
    limit/offset implementation.
    
    >>> Bug27Table(a=100).save()
    >>> Bug27Table(a=101).save()
    >>> len(list(Bug27Table.objects.all()))
    2
    
    >>> objs = list(Bug27Table.objects.extra(select={'alias_for_a':'a'}).order_by('a'))
    >>> objs[0].alias_for_a
    100

    >>> objs[1].alias_for_a
    101
    """
    
    a = models.IntegerField()

class Bug23Table(models.Model):
    """
    Test inserting mixed NULL and non-NULL values.

    >>> Bug23Table(mycharfield1=None, mycharfield2="text2", myintfield=1).save()
    >>> Bug23Table(mycharfield1="text1", mycharfield2=None, myintfield=1).save()
    >>> Bug23Table(mycharfield1="text1", mycharfield2="text2", myintfield=None).save()
    >>> Bug23Table(mycharfield1=None, mycharfield2=None, myintfield=1).save()
    >>> Bug23Table(mycharfield1=None, mycharfield2="text2", myintfield=None).save()
    >>> Bug23Table(mycharfield1="text1", mycharfield2=None, myintfield=None).save()
    >>> Bug23Table(mycharfield1=None, mycharfield2=None, myintfield=None).save()
    >>> objs = list(Bug23Table.objects.all())
    >>> len(objs)
    7
    >>> len([obj for obj in objs if obj.mycharfield1=="text1"])
    3
    >>> len([obj for obj in objs if obj.mycharfield2=="text2"])
    3
    >>> len([obj for obj in objs if obj.myintfield==1])
    3
    """
    mycharfield1 = models.CharField(max_length=100, null=True)
    mycharfield2 = models.CharField(max_length=50, null=True)
    myintfield = models.IntegerField(null=True)


# Bug 26 tables, RelatedA and RelatedB
class RelatedB(models.Model):
    a = models.CharField(max_length=50)
    b = models.CharField(max_length=50)
    c = models.CharField(max_length=50)

class RelatedA(models.Model):
    a = models.CharField(max_length=50)
    b = models.ForeignKey(RelatedB)


class Bug38Table(models.Model):
    d = models.DecimalField(max_digits=5, decimal_places=2)

class Bug34DatetimeTable(models.Model):
    """Ensure that __year filters work with datetime fields.
    
    >>> Bug34DatetimeTable(posted=datetime.date(2007,1,1)).save()
    >>> Bug34DatetimeTable(posted=datetime.date(2008,2,2)).save()
    >>> Bug34DatetimeTable(posted=datetime.date(2009,3,3)).save()
    >>> len((Bug34DatetimeTable.objects.filter(posted__day=3)))
    1
    >>> len(list(Bug34DatetimeTable.objects.filter(posted__month=2)))
    1
    >>> len(list(Bug34DatetimeTable.objects.filter(posted__year=2008)))
    1
    >>> len(list(Bug34DatetimeTable.objects.filter(posted__year=2005)))
    0
    """
    posted = models.DateTimeField()
        
class Bug34DateTable(models.Model):
    """Ensure that __year filters work with date fields.
    
    >>> Bug34DateTable(posted=datetime.date(2007,1,1)).save()
    >>> Bug34DateTable(posted=datetime.date(2008,2,2)).save()
    >>> Bug34DateTable(posted=datetime.date(2009,3,3)).save()
    >>> len((Bug34DateTable.objects.filter(posted__day=3)))
    1
    >>> len(list(Bug34DateTable.objects.filter(posted__month=2)))
    1
    >>> len(list(Bug34DateTable.objects.filter(posted__year=2008)))
    1
    >>> len(list(Bug34DateTable.objects.filter(posted__year=2005)))
    0
    """
    posted = models.DateField()

class Bug35ATable(models.Model):
    """Ensure that multiple Postive Integer columns across tables don't 
    create duplicate constraint names.
    """
    int1 = models.PositiveIntegerField()
    int2 = models.PositiveIntegerField()
    int3 = models.PositiveSmallIntegerField()
    int4 = models.PositiveSmallIntegerField()

class Bug35BTable(models.Model):
    """Ensure that multiple Postive Integer columns across tables don't 
    create duplicate constraint names.
    """
    int1 = models.PositiveIntegerField()
    int2 = models.PositiveIntegerField()
    int3 = models.PositiveSmallIntegerField()
    int4 = models.PositiveSmallIntegerField()

class Bug35CModel(models.Model):
    """Ensure that multiple Postive Integer columns across tables don't 
    create duplicate constraint names when using inheritence.
    
    The test for Bug35 is just the creation of the tables; there are no
    other explicit tests.
    """
    age = models.PositiveIntegerField()

    class Meta:
        abstract = True

class Bug35C1Table(Bug35CModel):
    name = models.CharField(max_length=10)
    
class Bug35C2Table(Bug35CModel):
    name = models.CharField(max_length=10)


class Bug37ATable(models.Model):
    a = models.CharField(max_length=50)
    b = models.CharField(max_length=50)
    c = models.CharField(max_length=50)

class Bug37BTable(models.Model):
    d = models.CharField(max_length=50)
    a = models.ForeignKey(Bug37ATable)

class Bug38Table(models.Model):
    d = models.DecimalField(max_digits=5, decimal_places=2)

# This probably should work but need a baseline of all tests passing for oraclepool for 0.6 release 
#class Bug41Table(models.Model):
#    """
#    Test that pagination works with extra/select columns.
#
#    >>> Bug41Table(a=100).save()
#    >>> Bug41Table(a=101).save()
#    >>> Bug41Table(a=102).save()
#    >>> len(list(Bug41Table.objects.all()))
#    3
#
#    >>> objs = Bug41Table.objects.extra(select={'alias_for_a':'[regress_bug41table].[a]'}).order_by('alias_for_a')
#    >>> all_objs = Paginator(objs, 1)
#    >>> all_objs.count
#    3
#    >>> all_objs.num_pages
#    3
#    >>> page1 = all_objs.page(1)
#    >>> page1.object_list[0].alias_for_a
#    100
#    >>> page2 = all_objs.page(2)
#    >>> page2.object_list[0].alias_for_a
#    101
#    """
#
#    a = models.IntegerField()

class Bug62Table(models.Model):
    email = models.CharField(max_length=255, blank=True)

class Bug58TableRecipe(models.Model):
    name = models.CharField(max_length=30)

class Bug58TableIn(models.Model):
    name = models.CharField(max_length=30)

class Bug58TableItem(models.Model):
    recipe = models.ForeignKey(Bug58TableRecipe, related_name='item')
    ingredient = models.ForeignKey(Bug58TableIn, related_name='item')
    amount = models.CharField(max_length=30)

class Bug66Table(models.Model):
    """
    Test that pagination works when pk uses db_column

    >>> Bug66Table(a=100).save()
    >>> Bug66Table(a=101).save()
    >>> Bug66Table(a=102).save()
    >>> len(list(Bug66Table.objects.all()))
    3

    >>> objs = Bug66Table.objects.all()
    >>> all_objs = Paginator(objs, 1)
    >>> all_objs.count
    3
    >>> all_objs.num_pages
    3
    >>> page1 = all_objs.page(1)
    >>> page1.object_list[0].a
    100
    >>> page2 = all_objs.page(2)
    >>> page2.object_list[0].a
    101
    """
    id = models.AutoField(primary_key=True, db_column='bug41id')
    a = models.IntegerField()
    
    



import settings
from django.db.backends.oracle.creation import DatabaseCreation
from django.test.simple import DjangoTestSuiteRunner
from datetime import datetime

# For repeated testing you may want to keep the test tables...
NO_TEARDOWN = False
# Monkeypatch standard oracle driver for tests 
# so it can be tested against an existing db too. See settings.
if settings.DATABASES['oracle']['EXTRAS'].get('existing',''):

    print 'Monkey patching for using existing database for tests'

    def _create_test_db(self, verbosity=1, autoclobber=False):
        """ if existing is set then this uses the settings database
            for testing rather than creating a new one
        """
        self.start_time(True)
        if not getattr(settings, 'TEST_DATABASE_NAME', ''):
            settings.TEST_DATABASE_NAME = settings.DATABASE_NAME
            settings.TEST_DATABASE_USER = settings.DATABASE_USER
            settings.TEST_DATABASE_PASSWD = settings.DATABASE_PASSWORD
        print 'Using Test Database %s' % settings.TEST_DATABASE_NAME
        return settings.TEST_DATABASE_NAME

    def start_time(self, reset=False):
        """ Getter / setter since attrib may be lost with monkey patch """
        if reset or not getattr(self, 'start', None):
            self.start = datetime.now()
        return self.start

    def _test_connection(self):
        """ Get the default connection from tests.settings """
        from django.db import connections, DEFAULT_DB_ALIAS
        return connections[DEFAULT_DB_ALIAS]

    def _destroy_test_db(self, test_database_name, verbosity=1):
        """ if existing is set then this must clean up all the test
            schema and data - not just drop the database
        """
        if hasattr(self, 'start_time'):
            print "#### Tested in %s ####" % str(datetime.now() - self.start_time())
        print 'Cleaning up test data and schema from %s' %  test_database_name
        from oraclepool.creation import DatabaseCreation as DCPool
        pool = DCPool(self._test_connection())
        if NO_TEARDOWN:
            print 'Leaving test tables around for speedy test repeat'
            pool._delete_test_data()
        else:
            print 'Dropping test tables'
            pool._drop_test_tables()
        pool._delete_test_users()

    def setup_databases(self, **kwargs):
        """ Dont create it, but do still run syncdb to populate test fixtures """
        connection = self._test_connection()
        alias = connection.alias
        from django.core.management import call_command
        try:
            call_command('syncdb',
                verbosity=1,
                interactive=False,
                database=alias,
                load_initial_data=False)
        except:
            # In case test suite cancelled before clean up in previous run
            _destroy_test_db(self, alias)
            call_command('syncdb',
                verbosity=1,
                interactive=False,
                database=alias,
                load_initial_data=False)
        return

    def dummy_teardown_databases(self, old_config, **kwargs):
        connection = self._test_connection()
        _destroy_test_db(self, connection.alias)

    DatabaseCreation.start_time = start_time
    DatabaseCreation._create_test_db = _create_test_db
    DatabaseCreation._destroy_test_db = _destroy_test_db
    setattr(DjangoTestSuiteRunner, '_test_connection', _test_connection)
    DjangoTestSuiteRunner.setup_databases = setup_databases
    DjangoTestSuiteRunner.teardown_databases = dummy_teardown_databases



from django.db import models

class BaseModel(models.Model):
    def __unicode__(self):
        return u'id: ' + unicode(self.id) + ' Amount: ' + unicode(self.val)

    class Meta:
        abstract = True

class TableNullTime(BaseModel):
    """
    This test isn't expected to work on SQL Server 2005,
    as there is no bare "time" type.
    
    TableNullTime(val=None).save()
    TableNullTime(val=datetime.time(2,34,2)).save()
    len(list(TableNullTime.objects.all()))
    2
    """
    val = models.TimeField(null=True)


class TableNullChar(BaseModel):
    val = models.CharField(null=True, max_length=100)
    
class TableNullText(BaseModel):
    val = models.TextField(null=True)

class TableNullInteger(BaseModel):
    val = models.IntegerField(null=True)

class TableNullDateTime(BaseModel):
    val = models.DateTimeField(null=True)

class TableNullDate(BaseModel):
    val = models.DateField(null=True)

class TableNullNullBoolean(BaseModel):
    val = models.NullBooleanField(null=True)

class TableNullDecimal(BaseModel):
    val = models.DecimalField(null=True, max_digits=4, decimal_places=2)

class TableNullFloat(BaseModel):
    val = models.FloatField(null=True)




from django.db import models

class OneTable(models.Model):
    b = models.CharField(max_length=100)
    c = models.CharField(default=u'test', max_length=10)
    
    def __repr__(self):
        return '<OneTable %s: %s, %s>' % (self.pk, self.b, self.c)


class TwoTable(models.Model):
    a = models.ForeignKey(OneTable)
    b = models.CharField(max_length=100)
    
    def __repr__(self):
        return '<TwoTable %s: %s>' % (self.pk, self.b)



# Middleware added if log level on INFO or DEBUG
# If dango DEBUG then write sql timings to screen else write to the log 
from django.conf import settings 
from django.db import connection
from django.template import Template, Context
from django.utils.html import strip_tags
logger = None
try:
    from oraclepool.base import get_extras
    from datetime import datetime
    if int(get_extras().get('log', 0)) == 10:
        import logging
        logger = logging.getLogger("oracle_pool")
except:
    pass

TMPL = Template('''
                <p>Session pool config: {{ pool }}</p> 
                <p><em>Total query count:</em> {{ count }}<br/>
                <em>Total execution time:</em> {{ time }}</p>
                <ul class="sqllog">
                    {% for sql in sqllog %}
                        <li>{{ sql.time }}: {{ sql.sql }}</li>
                    {% endfor %}
                </ul>
                ''')

class SQLLogMiddleware:
    """ Writes the pool details and sql query timings either to the log
        or at bottom of the page if django DEBUG is set
    """

    def process_response ( self, request, response ):
        """ SQL logging to screen switched on by settings.DEBUG
            SQL logging to log dependent on database logger level
            ie. EXTRAS['log'] = 10
            Use case is need to turn on debug SQL logging on production installs
            occasionally - whilst DEBUG is only for development
        """
        if settings.DEBUG or logger:
            time = 0.0
            for q in connection.queries:
                time += float(q['time'])

            if settings.DEBUG:
                if response.__getitem__('Content-Type').startswith('text/html'):

                    content = TMPL.render(Context({'pool':self.pool(),
                                                   'sqllog':connection.queries,
                                                   'count':len(connection.queries),
                                                   'time':time}))

                    if response.content.find('</body>') == -1:
                        response.content += content
                    else:
                        try:
                            response.content = response.content.replace('</body>',
                                                      u"%s%s" % (content,'</body>'))
                        except:
                            pass

            if logger:
                logger.debug('%s SQL log for %s -----------------------' % (datetime.now(),
                                                                            request.path) )
                logger.debug(self.pool())
                logger.debug('Executed %s queries in %s' % (len(connection.queries),time))
                for query in connection.queries:
                    logger.debug(query)
        return response


    def pool(self):
        """ show pooling info """
        if hasattr(connection, 'get_config'):
            return str(connection.get_config())
        else:
            return 'No connection pooling implemented'




"""
Oracle pooled connection database backend for Django.
Requires cx_Oracle: http://www.python.net/crew/atuining/cx_Oracle/
"""

import os
import thread
from django.db.backends import BaseDatabaseWrapper, BaseDatabaseValidation
try:
    from django.db.backends.signals import connection_created
except:
    connection_created = None
# Makes it explicit where the default oracle versions of these components are used
from django.db.backends.oracle.base import DatabaseFeatures as OracleDatabaseFeatures
from django.db.backends.oracle.base import DatabaseOperations as OracleDatabaseOperations
from django.db.backends.oracle.client import DatabaseClient as OracleDatabaseClient
from django.db.backends.oracle.introspection import DatabaseIntrospection as OracleDatabaseIntrospection
from django.db.backends.oracle.base import FormatStylePlaceholderCursor as OracleFormatStylePlaceholderCursor

from oraclepool.creation import DatabaseCreation
from django.conf import settings

try:
    import cx_Oracle as Database
except ImportError, e:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured("Error loading cx_Oracle module: %s" % e)

from django.utils.encoding import smart_str, force_unicode
# Check whether cx_Oracle was compiled with the WITH_UNICODE option.  This will
# also be True in Python 3.0.
if int(Database.version.split('.', 1)[0]) >= 5 and not hasattr(Database, 'UNICODE'):
    convert_unicode = force_unicode
else:
    convert_unicode = smart_str

# Oracle takes client-side character set encoding from the environment.
os.environ['NLS_LANG'] = '.UTF8'

def get_extras(user_defined_extras):
    """ Oracle already has OPTIONS specific to cx_Oracle.connection() use
        This adds extra pool and sql logging attributes to the settings 

        'homogeneous':1, # 1 = single credentials, 0 = multiple credentials
        Dropped this option to use multiple credentials since if supplied
        to Database.version (ie cx_Oracle) < '5.0.0' it breaks and we want
        separate pools for separate credentials anyhow.
    """
    default_extras = {'min':4,         # start number of connections
                      'max':8,         # max number of connections
                      'increment':1,   # increase by this amount when more are needed
                      'threaded':True, # server platform optimisation 
                      'timeout':600,   # connection timeout, 600 = 10 mins
                      'log':0,         # extra logging functionality
                      'logpath':'',    # file system path for oraclepool.log file
                      'existing':'',   # Type modifications if using existing database data
                      'like':'LIKEC',  # Use LIKE or LIKEC - Oracle ignores index for LIKEC on older dbs
                      'session':[]     # Add session optimisations applied to each fresh connection, eg.
                                       #   ['alter session set cursor_sharing = similar',
                                       #    'alter session set session_cached_cursors = 20']
                      }
    
    if user_defined_extras != None or len(user_defined_extras) != 0:
        return user_defined_extras
    elif hasattr(settings, 'DATABASE_EXTRAS'):
        return settings.DATABASE_EXTRAS
    else:
        return default_extras

def get_logger(extras):
    """ Check whether logging is required
        If log level is more than zero then logging is performed
        If log level is DEBUG then logging is printed to screen
        If no logfile is specified then unless its DEBUG to screen its added here
        NB: Log levels are 10 DEBUG, 20 INFO, 30 WARNING, 40 ERROR, 50 CRITICAL
    """

    loglevel = int(extras.get('log', 0))
    if loglevel > 0:
        import logging
        logfile = extras.get('logpath','')
        if logfile.endswith('.log'):
            (logfile, filename) = os.path.split(logfile)
        else:
            filename = 'oraclepool.log'
        if os.path.exists(logfile):
            logfile = os.path.join(logfile, filename)
        else:
            logfile = ''
        if not logfile and extras.get('log') > logging.DEBUG:
            logfile = '.'
        if logfile in ['.', '..']:
            logfile = os.path.join(os.path.abspath(os.path.dirname(logfile)), filename)
        # if log file is writable do it
        if not logfile:
            raise Exception('Log path %s not found' % extras.get('logpath', ''))
            return None
        else:
            logging.basicConfig(filename=logfile, level=loglevel)
            mylogger = logging.getLogger("oracle_pool")
            mylogger.setLevel(loglevel)
            chandler = logging.StreamHandler()
            chandler.setLevel(loglevel)
            fmt = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
            formatter = logging.Formatter(fmt)
            chandler.setFormatter(formatter)
            mylogger.addHandler(chandler)
            from datetime import datetime
            msg = '''%s #### Started django-oraclepool 
                     SQL logging at level %s ####''' % (datetime.now(), loglevel)
            mylogger.info(msg)
            return mylogger
    else:
        # 'No logging set'
        return None

    # Add sql logging for all requests if DEBUG level
    if extras.get('log') == 10 or settings.DEBUG:
        # Add middleware if needed
        middleware_classes = list(settings.MIDDLEWARE_CLASSES) 
        middleware_classes.append('oraclepool.log_sql.SQLLogMiddleware')
        settings.MIDDLEWARE_CLASSES = tuple(middleware_classes)
   
class DatabaseFeatures(OracleDatabaseFeatures):
    """ Add extra options from default Oracle ones
        Plus switch off save points and id return
        See 
        http://groups.google.com/group/django-developers/browse_thread/thread/bca33ecf27ff5d63
        Savepoints could be turned on but are not needed
        and since they may impact performance they are turned off here 
    """
    uses_savepoints = False
    can_return_id_from_insert = False
    allows_group_by_ordinal = False
    supports_tablespaces = True
    uses_case_insensitive_names = True
    time_field_needs_date = True
    date_field_supports_time_value = False

class DatabaseWrapper(BaseDatabaseWrapper):
    """ This provides the core connection object wrapper
        for cx_Oracle's pool handling.
        The code is mostly taken from
        http://code.djangoproject.com/ticket/7732 by halturin
    """

    poolprops = {'homogeneous':'',
                 'increment':'',
                 'max':'',
                 'min':'',
                 'busy':'',
                 'opened':'',
                 'name':'',
                 'timeout':'',
                 'tnsentry':''
                }
    operators = {
        'exact': '= %s',
        'iexact': '= UPPER(%s)',
        'contains': "LIKEC %s ESCAPE '\\'",
        'icontains': "LIKEC UPPER(%s) ESCAPE '\\'",
        'gt': '> %s',
        'gte': '>= %s',
        'lt': '< %s',
        'lte': '<= %s',
        'startswith': "LIKEC %s ESCAPE '\\'",
        'endswith': "LIKEC %s ESCAPE '\\'",
        'istartswith': "LIKEC UPPER(%s) ESCAPE '\\'",
        'iendswith': "LIKEC UPPER(%s) ESCAPE '\\'",
    }
    oracle_version = None

    def __init__(self, *args, **kwargs):
        """ Set up the various database components
            Oracle prefixed classes use the standard Oracle
            version - tested with both 1.0 and 1.2
        """
        super(DatabaseWrapper, self).__init__(*args, **kwargs)
        
        user_defined_extras = self.settings_dict['EXTRAS'] if 'EXTRAS' in self.settings_dict else {}
        self.extras = get_extras(user_defined_extras)
        self.logger = get_logger(self.extras)
        
        if self.extras.get('like', 'LIKEC') != 'LIKEC':
            for key in ['contains',
                        'icontains',
                        'startswith',
                        'istartswith',
                         'endswith',
                        'iendswith']:
                self.operators[key] = self.operators[key].replace('LIKEC', 
                                                     self.extras['like'])
        try:        
            self.features = DatabaseFeatures(self)
        except:
            # pre django 1.3
            self.features = DatabaseFeatures()
        try:
            self.ops = OracleDatabaseOperations(self)
        except:
            # pre django 1.4
            self.ops = OracleDatabaseOperations()
        self.client = OracleDatabaseClient(self)
        self.creation = DatabaseCreation(self)
        self.introspection = OracleDatabaseIntrospection(self)
        try:
            self.validation = BaseDatabaseValidation(self)
        except:
            # pre django 1.2
            self.validation = BaseDatabaseValidation()            

    def get_config(self):
        """ Report the oracle connection and pool data see 
            http://cx-oracle.sourceforge.net/html/session_pool.html#sesspool
        """
        pool = self._get_pool()
        if pool:
            for key in self.poolprops.keys():
                try:
                    self.poolprops[key] = getattr(pool, key, '')
                except:
                    pass
        else:
            self.poolprops['name'] = 'Session pool not found'
        return self.poolprops

    def _get_pool (self):
        """ Get the connection pool or create it if it doesnt exist
            Add thread lock to prevent server initial heavy load creating multiple pools
        """
        pool_name = '_pool_%s' % getattr(self, 'alias', 'common')
        if not hasattr (self.__class__, pool_name):
            lock = thread.allocate_lock()
            lock.acquire()
            if not hasattr (self.__class__, pool_name):            
                if self.extras['threaded']:
                    Database.OPT_Threading = 1
                else:
                    Database.OPT_Threading = 0
                # Use 1.2 style dict if its there, else make one
                try:
                    settings_dict = self.creation.connection.settings_dict
                except:
                    settings_dict = None

                if not settings_dict.get('NAME',''):
                    settings_dict = {'HOST':settings.DATABASE_HOST,
                                     'PORT':settings.DATABASE_PORT,
                                     'NAME':settings.DATABASE_NAME,
                                     'USER':settings.DATABASE_USER, 
                                     'PASSWORD':settings.DATABASE_PASSWORD, 
                                     }
                if len(settings_dict.get('HOST','').strip()) == 0:
                    settings_dict['HOST'] = 'localhost'
                if len(settings_dict.get('PORT','').strip()) != 0:
                    dsn = Database.makedsn(settings_dict['HOST'],
                                           int(settings_dict['PORT']),
                                           settings_dict.get('NAME',''))
                else:
                    dsn = settings_dict.get('NAME','')

                try:
                    pool = Database.SessionPool(settings_dict.get('USER',''), 
                                                settings_dict.get('PASSWORD',''), 
                                                dsn, 
                                                self.extras.get('min',4), 
                                                self.extras.get('max',8), 
                                                self.extras.get('increment',1),
                                                threaded = self.extras.get('threaded',
                                                                               True))
                except Exception, err:
                    pool = None
                if pool:
                    if self.extras.get('timeout', 0):
                        pool.timeout = self.extras['timeout']
                    setattr(self.__class__, pool_name, pool)
                else:
                    msg = """##### Database '%(NAME)s' login failed or database not found ##### 
                             Using settings: %(USER)s @ %(HOST)s:%(PORT)s / %(NAME)s  
                             Django start up cancelled
                          """ % settings_dict
                    msg += '\n##### DUE TO ERROR: %s\n' % err
                    if self.logger:
                        self.logger.critical(msg)
                    else:
                        print msg
                    return None
                lock.release()
        return getattr(self.__class__, pool_name)
        
    pool = property(_get_pool)

    def _cursor(self, settings=None):
        """ Get a cursor from the connection pool """
        cursor = None
        if self.pool is not None:
            if self.connection is None:
                # Set oracle date to ansi date format.  This only needs to execute
                # once when we create a new connection. 
                self.connection = self.pool.acquire()
                if connection_created:
                    # Assume acquisition of existing connection = create for django signal
                    connection_created.send(sender=self.__class__)
                if self.logger:
                    self.logger.info("Acquire pooled connection \n%s\n" % self.connection.dsn)

                cursor = FormatStylePlaceholderCursor(self.connection)
                cursor.set_logger(self.logger)
                cursor.execute("ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD' "  
                               "NLS_TIMESTAMP_FORMAT = 'YYYY-MM-DD HH24:MI:SS.FF'")
                if self.extras.get('session', []):
                    for sql in self.extras['session']:
                        cursor.execute(sql)
                try:
                    self.oracle_version = int(self.connection.version.split('.')[0])
                    # There's no way for the DatabaseOperations class to know the
                    # currently active Oracle version, so we do some setups here.
                    # TODO: Multi-db support will need a better solution (a way to
                    # communicate the current version).
                    if self.oracle_version <= 9:
                        self.ops.regex_lookup = self.ops.regex_lookup_9
                    else:
                        self.ops.regex_lookup = self.ops.regex_lookup_10
                except ValueError, err:
                    if self.logger:
                        self.logger.warn(str(err))
                try:
                    self.connection.stmtcachesize = 20
                except:
                    # Django docs specify cx_Oracle version 4.3.1 or higher, but
                    # stmtcachesize is available only in 4.3.2 and up.
                    pass
            else:
                cursor = FormatStylePlaceholderCursor(self.connection)                
                cursor.set_logger(self.logger)
        else:
            if self.logger:
                self.logger.critical('Pool couldnt be created - please check your Oracle connection or credentials')
            else:
                raise Exception('\n\n##### Pool couldnt be created - please check your Oracle connection or credentials')
            
        if not cursor:
            cursor = FormatStylePlaceholderCursor(self.connection)
            cursor.set_logger(self.logger)
        # Default arraysize of 1 is highly sub-optimal.
        cursor.arraysize = 100
        return cursor

    def close(self):
        """ Releases connection back to pool """
        if self.connection is not None:
            if self.logger:
                self.logger.debug("Release pooled connection\n%s\n" % self.connection.dsn)
            try:
                self.pool.release(self.connection)
            except Database.OperationalError as error:
                if self.logger:
                    self.logger.debug("Release pooled connection failed due to: %s" % str(error))
            finally:
                self.connection = None

    def _savepoint_commit(self, sid):
        """ Oracle doesn't support savepoint commits.  Ignore them. """
        pass

class FormatStylePlaceholderCursor(OracleFormatStylePlaceholderCursor):
    """ Added just to allow use of % for like queries without params
        and use of logger if present.
    """

    logger = None

    def set_logger(self, logger):
        """ Added to assign logger as cursor property - to make its move from 
            the global namespace in B.Watson's code work OK
        """
        self.logger = logger

    def cleanquery(self, query, args=None):
        """ cx_Oracle wants no trailing ';' for SQL statements.  For PL/SQL, it
            it does want a trailing ';' but not a trailing '/'.  However, these
            characters must be included in the original query in case the query
            is being passed to SQL*Plus.

            Split out this as a function and allowed for no args so
            % signs can be used in the query without requiring parameterization
        """
#        if query.find('INSERT')> -1:
#            raise Exception(query) #params[8])
        if query.endswith(';') or query.endswith('/'):
            query = query[:-1]
        if not args:
            return convert_unicode(query, self.charset)
        else:
            try:
                return convert_unicode(query % tuple(args), self.charset)
            except TypeError, error:
                err = 'Parameter parsing failed due to error %s for query: %s' % (error,
                                                                                  query)
                if self.logger:
                    self.logger.critical(err)
                else:
                    raise Exception(err)
                
    def execute(self, query, params=[]):
        if params is None:
            args = None
        else:
            params = self._format_params(params)
            args = [(':arg%d' % i) for i in range(len(params))]
        query = self.cleanquery(query, args)
        self._guess_input_sizes([params])
        try:
            return self.cursor.execute(query, self._param_generator(params))
        except Database.Error, error:
            # cx_Oracle <= 4.4.0 wrongly raises a Database.Error for ORA-01400.
            if error.args[0].code == 1400 and not isinstance(error, 
                                                             Database.IntegrityError):
                error = Database.IntegrityError(error.args[0])
            err = '%s due to query:%s' % (error, query)
            if self.logger:
                self.logger.critical(err)
            else:
                raise Exception(err) 

    def executemany(self, query, params=[]):
        try:
            args = [(':arg%d' % i) for i in range(len(params[0]))]
        except (IndexError, TypeError):
            # No params given, nothing to do
            return None
        query = self.cleanquery(query, args)        
        formatted = [self._format_params(i) for i in params]
        self._guess_input_sizes(formatted)
        try:
            return self.cursor.executemany(query,
                                [self._param_generator(p) for p in formatted])
        except Database.Error, error:
            # cx_Oracle <= 4.4.0 wrongly raises a Database.Error for ORA-01400.
            if error.args[0].code == 1400 and not isinstance(error, 
                                                             Database.IntegrityError):
                error = Database.IntegrityError(error.args[0])
            if self.logger:
                self.logger.critical('%s due to query: %s' % (error, query))                
            else:
                raise 



from django.db import models
from django.core.paginator import Paginator
from django.test import TestCase


class FirstTable(models.Model):
    b = models.CharField(max_length=100)
    # Add a reserved word column; this will get quoted correctly
    # in queries, but need to make sure paging doesn't break:
    # * Paging should re-quote alias names correctly
    # * The string splitting on 'FROM' shouldn't break either
    c = models.CharField(default=u'test', max_length=10, db_column=u'FROM')
    
    def __repr__(self):
        return '<FirstTable %s: %s, %s>' % (self.pk, self.b, self.c)


class SecondTable(models.Model):
    a = models.ForeignKey(FirstTable)
    b = models.CharField(max_length=100)
    
    def __repr__(self):
        return '<FirstTable %s: %s, %s>' % (self.pk, self.a_id, self.b)


class Products(models.Model):
    """
    >>> names=['D', 'F', 'B', 'A', 'C', 'E', 'G']
    >>> for n in names: product = Products.objects.create(name=n)
    >>> p = Products.objects
    >>> len(list(p.all()))
    7
    >>> len(list(p.all()[:3]))
    3
    >>> len(list(p.all()[2:5]))
    3
    >>> len(list(p.all()[5:]))
    2
    >>> p.all()[0:0]
    []
    >>> p.all()[0:0][:10]
    []
    >>> pn = p.order_by('name')
    >>> list(pn)
    [A, B, C, D, E, F, G]
    >>> list(pn[:3])
    [A, B, C]
    >>> list(pn[2:5])
    [C, D, E]
    >>> list(pn[5:])
    [F, G]
    """

    productid = models.AutoField(primary_key=True)
    name = models.CharField(max_length=50)
        
    def __repr__(self):
        return self.name
        
    def __unicode__(self):
        return "<Product %u: %u>" % (self.productid, self.name)

        
class PagingTestCase(TestCase):
    """The Paginator uses slicing internally."""
    fixtures = ['paging.json']
    
    def get_q(self, a1_pk):
        return SecondTable.objects.filter(a=a1_pk).order_by('b').select_related(depth=1)

    def try_page(self, page_number, q):
        # Use a single item per page, to get multiple pages.
        pager = Paginator(q, 1)
        self.assertEquals(pager.count, 3)

        on_this_page = list(pager.page(page_number).object_list)
        self.assertEquals(len(on_this_page), 1)
        self.assertEquals(on_this_page[0].b, 'B'+str(page_number))
    
    def testWithDuplicateColumnNames(self):
        a1_pk = FirstTable.objects.get(b='A1').pk
        q = self.get_q(a1_pk)
        
        for i in (1,2,3):
            self.try_page(i, q)
            
    def testPerRowSelect(self):
        a1_pk = FirstTable.objects.get(b='A1').pk
        q = SecondTable.objects.filter(a=a1_pk).order_by('b').select_related(depth=1).extra(select=
        {
        'extra_column': 
            "select slicing_FirstTable.id from slicing_FirstTable where slicing_FirstTable.id=%s" % (a1_pk,)
        })
        
        for i in (1,2,3):
            self.try_page(i, q)


class DistinctTable(models.Model):
    """ Slicing tests require ordered result set """
    s = models.CharField(max_length=10)
    class Meta:
        ordering = ['s',]

class DistinctTestCase(TestCase):
    def testLimitDistinct(self):
        T = DistinctTable
        T(s='abc').save()
        T(s='abc').save()
        T(s='abc').save()
        T(s='def').save()
        T(s='def').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='ijk').save()
        T(s='ijk').save()
        T(s='xyz').save()
        
        baseQ = T.objects.values_list('s', flat=True).distinct()
        
        stuff = list(baseQ)
        self.assertEquals(len(stuff), 5)
        
        stuff = list(baseQ[:2])
        self.assertEquals(stuff, [u'abc', u'def'])

        stuff = list(baseQ[3:])
        self.assertEquals(stuff, [u'ijk', u'xyz'])

        stuff = list(baseQ[2:4])
        self.assertEquals(stuff, [u'fgh', u'ijk'])

    def testUnusedDistinct(self):
        T = DistinctTable
        T(s='abc').save()
        T(s='abc').save()
        T(s='abc').save()
        T(s='def').save()
        T(s='def').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='ijk').save()
        T(s='ijk').save()
        T(s='xyz').save()
        
        baseQ = T.objects.distinct()

        stuff = list(baseQ)
        self.assertEquals(len(stuff), 12)

        stuff = list(baseQ[:2])
        self.assertEquals(
            [o.s for o in stuff],
            [u'abc', u'abc'])

        stuff = list(baseQ[3:])
        self.assertEquals(
            [o.s for o in stuff], 
            [u'def', u'def', u'fgh', u'fgh', u'fgh', u'fgh', u'ijk', u'ijk', u'xyz'])

        stuff = list(baseQ[2:4])
        self.assertEquals(
            [o.s for o in stuff], 
            [u'abc', u'def'])



import decimal
from django.test import TestCase
from oraclepool.tests.regress.models import *

class Bug26TestCase(TestCase):
    """Test that slicing queries w/ duplicate column names works."""

    def testWithDuplicateColumnNames(self):
        b = RelatedB(a='this is a value', b="valueb", c="valuec")
        b.save()
        
        RelatedA(a="valuea", b=b).save()
        RelatedA(a="valuea", b=b).save()

        items = RelatedA.objects.select_related()[1:2]
        self.assertEqual(len(items), 1)

class Bug37TestCase(TestCase):
    """Test that IntegrityErrors are raised when appropriate."""

    def testDuplicateKeysFails(self):
        Bug37ATable(pk=1, a='a', b='b', c='c').save(force_insert=True)
        try:
            Bug37ATable(pk=1, a='a', b='b', c='c').save(force_insert=True)
        except Exception, e:
            self.failUnless(isinstance(e, IntegrityError) or str(e).find('ORA-00001')>-1,
                            'Expected IntegrityError but got: %s - %s' % (type(e),str(e)))
            
    def testDeleteRelatedRecordFails(self):
        a2 = Bug37ATable(a='a', b='b', c='c')
        a2.save()
        
        Bug37BTable(d='d', a=a2).save()
        try:
            a2.delete()
        except Exception, e:
            self.failUnless(isinstance(e, IntegrityError), 'Expected IntegrityError but got: %s' % type(e))

class Bug38TestCase(TestCase):
    def testInsertVariousFormats(self):
        """
        Test adding decimals as strings with various formats.
        """
        Bug38Table(d=decimal.Decimal('0')).save()
        Bug38Table(d=decimal.Decimal('0e0')).save()
        Bug38Table(d=decimal.Decimal('0E0')).save()
        Bug38Table(d=decimal.Decimal('450')).save()
        Bug38Table(d=decimal.Decimal('450.0')).save()
        Bug38Table(d=decimal.Decimal('450.00')).save()
        Bug38Table(d=decimal.Decimal('450.000')).save()
        Bug38Table(d=decimal.Decimal('0450')).save()
        Bug38Table(d=decimal.Decimal('0450.0')).save()
        Bug38Table(d=decimal.Decimal('0450.00')).save()
        Bug38Table(d=decimal.Decimal('0450.000')).save()
        Bug38Table(d=decimal.Decimal('4.5e+2')).save()
        Bug38Table(d=decimal.Decimal('4.5E+2')).save()
        self.assertEquals(len(list(Bug38Table.objects.all())),13)

    def testReturnsDecimal(self):
        """
        Test if return value is a python Decimal object 
        when saving the model with a Decimal object as value 
        """
        Bug38Table(d=decimal.Decimal('0')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal, d1.d.__class__)

    def testReturnsDecimalFromString(self):
        """
        Test if return value is a python Decimal object 
        when saving the model with a unicode object as value.
        """
        Bug38Table(d=u'123').save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal, d1.d.__class__)        

    def testSavesAfterDecimal(self):
        """
        Test if value is saved correctly when there are numbers 
        to the right side of the decimal point 
        """
        Bug38Table(d=decimal.Decimal('450.1')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal('450.1'), d1.d)
    
    def testInsertWithMoreDecimals(self):
        """
        Test if numbers to the right side of the decimal point 
        are saved correctly rounding to a decimal with the correct 
        decimal places.
        """
        Bug38Table(d=decimal.Decimal('450.111')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal('450.11'), d1.d)    
        
    def testInsertWithLeadingZero(self):
        """
        Test if value is saved correctly with Decimals with a leading zero.
        """
        Bug38Table(d=decimal.Decimal('0450.0')).save()
        d1 = Bug38Table.objects.all()[0]
        self.assertEquals(decimal.Decimal('450.0'), d1.d)


class Bug58TestCase(TestCase):
    def testDistinctRelated(self):
        i1 = Bug58TableIn(name='bread')
        i1.save()
        i2 = Bug58TableIn(name='butter')
        i2.save()
        
        r = Bug58TableRecipe(name='toast')
        r.save()
        
        Bug58TableItem(recipe=r, ingredient=i1, amount='1 slice').save()
        Bug58TableItem(recipe=r, ingredient=i2, amount='1 Tbsp').save()

        q = list(Bug58TableRecipe.objects.filter(item__ingredient__name__in=['bread','butter']).distinct())
        self.assertEqual(len(q), 1)

        



from django.core.paginator import Paginator
from django.test import TestCase
from oraclepool.tests.slicing.models import *
        
class PagingTestCase(TestCase):
    """The Paginator uses slicing internally."""
    fixtures = ['paging.json']
    
    def get_q(self, a1_pk):
        return SecondTable.objects.filter(a=a1_pk).order_by('b').select_related(depth=1)

    def try_page(self, page_number, q):
        # Use a single item per page, to get multiple pages.
        pager = Paginator(q, 1)
        self.assertEquals(pager.count, 3)

        on_this_page = list(pager.page(page_number).object_list)
        self.assertEquals(len(on_this_page), 1)
        self.assertEquals(on_this_page[0].b, 'B'+str(page_number))
    
    def testWithDuplicateColumnNames(self):
        a1_pk = FirstTable.objects.get(b='A1').pk
        q = self.get_q(a1_pk)
        
        for i in (1,2,3):
            self.try_page(i, q)
            
    def testPerRowSelect(self):
        a1_pk = FirstTable.objects.get(b='A1').pk
        q = SecondTable.objects.filter(a=a1_pk).order_by('b').select_related(depth=1).extra(select=
        {
        'extra_column': 
            "select slicing_FirstTable.id from slicing_FirstTable where slicing_FirstTable.id=%s" % (a1_pk,)
        })
        
        for i in (1,2,3):
            self.try_page(i, q)

class DistinctTestCase(TestCase):
    def testLimitDistinct(self):
        T = DistinctTable
        T(s='abc').save()
        T(s='abc').save()
        T(s='abc').save()
        T(s='def').save()
        T(s='def').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='ijk').save()
        T(s='ijk').save()
        T(s='xyz').save()
        
        baseQ = T.objects.values_list('s', flat=True).distinct()
        
        stuff = list(baseQ)
        self.assertEquals(len(stuff), 5)
        
        stuff = list(baseQ[:2])
        self.assertEquals(stuff, [u'abc', u'def'])

        stuff = list(baseQ[3:])
        self.assertEquals(stuff, [u'ijk', u'xyz'])

        stuff = list(baseQ[2:4])
        self.assertEquals(stuff, [u'fgh', u'ijk'])

    def testUnusedDistinct(self):
        T = DistinctTable
        T(s='abc').save()
        T(s='abc').save()
        T(s='abc').save()
        T(s='def').save()
        T(s='def').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='fgh').save()
        T(s='ijk').save()
        T(s='ijk').save()
        T(s='xyz').save()
        
        baseQ = T.objects.distinct()

        stuff = list(baseQ)
        self.assertEquals(len(stuff), 12)

        stuff = list(baseQ[:2])
        self.assertEquals(
            [o.s for o in stuff],
            [u'abc', u'abc'])

        stuff = list(baseQ[3:])
        self.assertEquals(
            [o.s for o in stuff], 
            [u'def', u'def', u'fgh', u'fgh', u'fgh', u'fgh', u'ijk', u'ijk', u'xyz'])

        stuff = list(baseQ[2:4])
        self.assertEquals(
            [o.s for o in stuff], 
            [u'abc', u'def'])



from django.conf import settings
from django.db.backends.oracle.creation import DatabaseCreation as OracleDatabaseCreation
data_types = OracleDatabaseCreation.data_types
import sys
from datetime import datetime

def existing(settings_dict):
    """ Use existing database for tests? 
        Cater for using existing Oracle databases both for format issues and
        for database tests where you dont have oracle dba drop database permissions
        Eg: if existing = 'Unicode' then its just used for the latter.
    """
    extras_setting = settings_dict['EXTRAS'] if 'EXTRAS' in settings_dict else None
    
    if extras_setting.has_key('existing'):
        return extras_setting['existing']
    elif hasattr(settings, 'DATABASE_EXTRAS'):
        return settings.DATABASE_EXTRAS.get('existing','')
    else:
        return ''

# Uses the specified encoding
def get_ascii_data_types(settings_dict): 
    if existing(settings_dict) == 'ASCII':
        # Default CharFields are not Unicode
        data_types['CharField'] = 'VARCHAR2(%(max_length)s)'
        data_types['FileField'] = 'VARCHAR2(%(max_length)s)'
        data_types['FilePathField'] = 'VARCHAR2(%(max_length)s)'
        return data_types
    else:
        return None

class DatabaseCreation(OracleDatabaseCreation):
    """ Allow for modification of the data types to cater for using
        older or existing Oracle database data. 
    """
    start = datetime.now()

    def __init__(self, connection):
        OracleDatabaseCreation.__init__(self, connection)
        
        ascii_data_types = get_ascii_data_types(self.connection.settings_dict)
        if ascii_data_types != None:
            data_types = ascii_data_types
    
    def _create_test_db(self, verbosity=1, autoclobber=False):
        """ If existing is set then this uses the settings database
            for testing rather than creating a new one
        """
        self.start = datetime.now()

        # 'Option for using existing (non-production) database for tests' 
        if existing(self.connection.settings_dict):
            conn_settings = self.connection.settings_dict
            if conn_settings.has_key('TEST_NAME'):
                if not conn_settings['TEST_NAME']:
                    conn_settings['TEST_NAME'] = conn_settings['NAME']
            # django pre 1.3 global settings
            elif hasattr(settings, 'TEST_DATABASE_NAME') and not settings.TEST_DATABASE_NAME:
                settings.TEST_DATABASE_NAME = settings.DATABASE_NAME
                settings.TEST_DATABASE_USER = settings.DATABASE_USER
                settings.TEST_DATABASE_PASSWD = settings.DATABASE_PASSWORD
        else:
            super(OracleDatabaseCreation, self)._create_test_db(verbosity=verbosity,
                                                                autoclobber=autoclobber)
        test_db = conn_settings.get('TEST_NAME', getattr(settings, 'TEST_DATABASE_NAME', 'None'))
        print 'Using Test Database %s' % test_db


    def _destroy_test_db(self, test_database_name, verbosity=1):
        """ If existing is set then this must clean up all the test
            schema and data - not just drop the database
        """
        print "#### Built tables and tested in %s ####" % str(datetime.now() - self.start)
        if existing(self.connection.settings_dict):
            print 'Cleaning up test data and schema from %s' % settings.TEST_DATABASE_NAME
            self._drop_test_tables()
            self._delete_test_users()
        else:
            print 'Destroying Test Database %s' % settings.TEST_DATABASE_NAME            
            super(OracleDatabaseCreation, self)._destroy_test_db(verbosity=verbosity,
                                                                 autoclobber=False)
        
    def list_test_tables(self, apps=None):
        """ Only used when using an existing database for testing. 
            Retrieve the list of models that the tests create
            so they can be dropped again rather than blitzing the whole db
            NB: This assumes running the tests via the separate tests/manage.py
            Where all the apps are test apps - otherwise specify the test apps
        """
        from django.db import models
        test_tables = []
        tables = self.connection.introspection.table_names()        
        if not apps:
            apps = models.get_apps()
        for app in apps:
            app_name = app.__name__.split('.')[-2]
            model_list = models.get_models(app)
            for model in model_list:
                table = self.connection.introspection.table_name_converter(model._meta.db_table)
                if table in tables:
                    test_tables.append(table)
        return test_tables

    def _drop_test_tables(self):
        """ Individually drop the test tables """
        from django.db import connection, transaction
        cursor = connection.cursor()
        try:
            for table in self.list_test_tables():
                statement = "drop table " + table + " cascade constraints purge"
                try:
                    cursor.execute(statement)
                except:
                    pass
                statement = "drop sequence " + table + "_SQ"
                try:
                    cursor.execute(statement)            
                except:
                    pass
                print 'Deleted table and sequence %s' % table
            transaction.commit_unless_managed()            
        except Exception, err:
            print 'Couldnt acquire transaction to delete test tables due to error: %s' % err
        return

    def _delete_test_data(self):
        """ Individually delete the test tables """
        from django.db import connection, transaction
        cursor = connection.cursor()
        try:
            for table in self.list_test_tables():
                statement = "delete from " + table 
                try:
                    cursor.execute(statement)
                except:
                    pass
                try:
                    cursor.execute(statement)            
                except:
                    pass
            print 'Deleted test data'
            transaction.commit_unless_managed()            
        except Exception, err:
            print 'Couldnt acquire transaction to delete test data due to error: %s' % err
        return

    def _delete_test_users(self):
        """ individually delete the test users """
        user_clause =  " from auth_user where email like '%@example.com' or email is null"
        try:
            from django.db import connection, transaction
            cursor = connection.cursor()
            cursor.execute("delete from django_admin_log where user_id in (select id " + user_clause + ')')
            cursor.execute("delete " + user_clause)
            transaction.commit_unless_managed()
            print 'Deleted test users'
        except Exception, err:
            print 'Couldnt acquire connection to delete test users due to %s' % err
        return



import datetime
import decimal
from django.test import TestCase

from oraclepool.tests.nulls.models import *

class NullTests(TestCase):
    def _run(self, model, values):
        for v in values:
            model(val=v).save()
            
        self.assertEquals(len(list(model.objects.all())), len(values))

    def testChar(self):
        self._run(TableNullChar, (None, "This is my string value."))
        
    def testText(self):
        self._run(TableNullText, (None, "This is my string value."))
        
    def testInteger(self):
        self._run(TableNullInteger, (None, 32768))
        
    def testDateTime(self):
        self._run(TableNullDateTime,(None, datetime.datetime(2009,1,1,4,3,5)))
     
    def testDate(self):
        self._run(TableNullDate, (None, datetime.date(2009,1,1)))
     
    def testBoolean(self):
        self._run(TableNullNullBoolean, (None, True, False))

    def testFloat(self):
        self._run(TableNullFloat, (None, 34.3))

    def testDecimal(self):
        self._run(TableNullDecimal, (None, decimal.Decimal('99.99')))



# Django settings for testbackend project.
# from dbsettings import *

# Set up common credentials for testing
# oraclepool vs. oracle functionality & performance 
try:
    from cred import CREDENTIALS
except:
    CREDENTIALS = {'USER' : 'SCOTT',      
                   'NAME' : 'SCOTT',             
                   'PASSWORD' : 'TIGER',     
                   'HOST' : 'localhost', 
                   'PORT' : '1521',            
                   'OPTIONS' : {}
                   }

# NB: Oracle already has OPTIONS dict so use EXTRAS for pooling and logging
EXTRAS = {'min':4,         # start number of connections
          'max':8,         # max number of connections
          'increment':2,   # increase by this amount when more are needed
          'homogeneous':1, # 1 = single credentials, 0 = multiple credentials
          'threaded':True, # server platform optimisation 
          'timeout':10,   # connection timeout, 600 = 10 mins
          'log':0,         # extra logging functionality
          'logpath':'.',    # file system path to log file
          'existing':'Unicode',   # Type modifications if using existing database data
          'session': ["alter session set session_cached_cursors = 8;",
                      "alter session set cursor_sharing = 'SIMILAR'"]
          }

# Use django 1.2 or later multi-db setting (see rewrite for pre 1.2 below ...)
DATABASES= { 'oraclepool':{'ENGINE' : 'oraclepool',
                           'EXTRAS' : EXTRAS
                          },
             'oracle':{'ENGINE' : 'django.db.backends.oracle',
                       'EXTRAS' : {'existing':'Unicode'}
                       }
             }

for db in ['oraclepool','oracle']:
    for key in CREDENTIALS.keys():
        DATABASES[db][key] = CREDENTIALS[key] 

### Switch backend for standard tests
### Specify oracle or oraclepool for running the test suite against
DATABASES['default'] = DATABASES['oraclepool']

def get_settings_dict(db='default'):
    """ Make compatible with 1.2 or earlier database settings
        used by apitest and performance test
    """   
    try:
        from settings import DATABASES
        settings_dict = DATABASES.get(db,{})
    except:
        pass
    if not settings_dict:
        for key in ['HOST','PORT','NAME','USER','PASSWORD','ENGINE','OPTIONS','EXTRAS']:
            settings_dict[key] = globals().get('DATABASE_%s' % key, '') 
    for key in settings_dict.keys():
        settings_dict['DATABASE_%s' % key] = settings_dict[key]
    return settings_dict

# Also rewrite into globals for pre django 1.2 compatibility
for k, v in DATABASES['default'].items():
    globals()['DATABASE_' + k] = v

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/New_York'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = ''

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
STATIC_URL = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = '%(u8uds4xsy0+95cj3o&k49*u@&--yp0t&e&0$!@s2fvea#u4j'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.doc.XViewMiddleware',
)

ROOT_URLCONF = 'urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'oraclepool',
    'oraclepool.tests.performance',
    'oraclepool.tests.apitest',
    'oraclepool.tests.regress',
    'oraclepool.tests.slicing',
    'oraclepool.tests.nulls',
    'oraclepool.tests.aggregates'
)
